package com.ims.eval.controller;

import cn.hutool.core.convert.Convert;
import cn.hutool.core.util.IdUtil;
import cn.hutool.core.util.RandomUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.ims.common.utils.StringUtils;
import com.ims.eval.config.CustomException;
import com.ims.eval.entity.*;
import com.ims.eval.entity.dto.result.R;
import com.ims.eval.service.*;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.util.Arrays;
import java.util.Iterator;
import java.util.List;

/**
 * 本部部门业绩指标计划
 *
 * @author hlf
 * @date 2023/4/26 14:15
 * 文件说明：
 */
@RestController
@RequestMapping("//evaluation-dept-plan")
public class DepartmentalPerformanceIndicatorPlanController {

	@Autowired
	private IEvaluationDeptBusinessPlanService evaluationDeptBusinessPlanService;

	@Autowired
	private IEvaluationDeptPlanContentService evaluationDeptPlanContentService;

	@Autowired
	private HttpServletRequest request;

	/**
	 * 部门考评业务列表信息（分页）
	 *
	 * @param pageNum            当前记录起始索引
	 * @param pageSize           每页显示记录数
	 * @param responsibilityCode 业务编码
	 * @param des                业务简述
	 * @param appraisalYear      月度
	 * @param stage              流程状态
	 * @return 结果
	 */
	//@ImsPreAuth("eval:evaluationDeptBusinessPlan:view")
	@GetMapping(value = "list")
	public R list(
		@RequestParam(value = "pageNum") Integer pageNum,
		@RequestParam(value = "pageSize") Integer pageSize,
		@RequestParam(value = "responsibilityCode", required = false) String responsibilityCode,
		@RequestParam(value = "des", required = false) String des,
		@RequestParam(value = "appraisalYear", required = false) String appraisalYear,
		@RequestParam(value = "stage", required = false) String stage) {
		IPage<EvaluationDeptBusinessPlan> list = evaluationDeptBusinessPlanService.listPage(pageNum, pageSize, responsibilityCode, des, appraisalYear, stage);
		return R.ok().data(list);
	}

	/**
	 * 部门考评业务信息
	 *
	 * @param responsibilityCode 业务编码
	 * @param des                业务简述
	 * @param appraisalYear      月度
	 * @return 结果
	 */
	//@ImsPreAuth("eval:evaluationDeptBusinessPlan:view")
	@GetMapping(value = "listAll")
	public R listAll(
		@RequestParam(value = "responsibilityCode", required = false) String responsibilityCode,
		@RequestParam(value = "des", required = false) String des,
		@RequestParam(value = "appraisalYear", required = false) String appraisalYear) {
		List<EvaluationDeptBusinessPlan> list = evaluationDeptBusinessPlanService.listAll(responsibilityCode, des, appraisalYear);
		return R.ok().data(list);
	}

	/**
	 * 新增部门考评业务信息
	 *
	 * @param deptBusinessPlan 部门考评业务实体
	 * @return 结果
	 */
	//@ImsPreAuth("eval:evaluationDeptBusinessPlan:edit")
	@PostMapping(value = "/save")
	public R addAll(@RequestBody EvaluationDeptBusinessPlan deptBusinessPlan) {
		try {
			boolean b = evaluationDeptBusinessPlanService.save(deptBusinessPlan);
			if (b) {
				return R.ok().data(b);
			} else {
				return R.error().data("保存失败！");
			}
		} catch (CustomException e) {
			return R.customError(e.getMessage()).data("失败！");
		}
	}

	/**
	 * 批量删除部门考评业务信息
	 *
	 * @param ids
	 * @return
	 */
	//@ImsPreAuth("eval:evaluationDeptBusinessPlan:remove")
	@PostMapping(value = "/removeAll/{ids}")
	public R deleteAll(@PathVariable("ids") String ids) {
		String[] strings = ids.split(",");
		boolean b = evaluationDeptBusinessPlanService.removeByIds(Arrays.asList(strings));
		if (b) {
			return R.ok().data(b);
		} else {
			return R.error().data("删除失败！");
		}
	}

	/**
	 * 详情
	 *
	 * @param id 业务主键
	 * @return 结果
	 */
	//@ImsPreAuth("eval:evaluationDeptBusinessPlan:view")
	@GetMapping(value = "/details/{id}")
	public R details(@PathVariable("id") String id) {
		List<EvaluationDeptPlanContent> list = evaluationDeptPlanContentService.detailsList(id, "指标计划", request);
		return R.ok().data(list);
	}

	/**
	 * 详情里指标内容修改
	 *
	 * @param evaluationDeptPlanContentList
	 * @return
	 */

	//@ImsPreAuth("eval:evaluationDeptBusinessPlan:edit")
	@PostMapping(value = "/update")
	public R update(@RequestBody List<EvaluationDeptPlanContent> evaluationDeptPlanContentList) {
		try {
			boolean b = evaluationDeptPlanContentService.saveOrUpdateBatch(evaluationDeptPlanContentList);
			if (b) {
				return R.ok().data(b);
			} else {
				return R.error().data("修改失败！");
			}
		} catch (CustomException e) {
			return R.customError(e.getMessage()).data("失败！");
		}
	}
}
