package com.ims.eval.service.impl;

import cn.hutool.core.convert.Convert;
import cn.hutool.core.util.IdUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.ims.common.utils.StringUtils;
import com.ims.eval.dao.EvaluationDeptBusinessPlanMapper;
import com.ims.eval.entity.*;
import com.ims.eval.service.*;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;

/**
 * @author hlf
 * @date 2023/4/27 10:33
 * 文件说明：
 */
@Service
public class EvaluationDeptBusinessPlanServiceImpl extends ServiceImpl<EvaluationDeptBusinessPlanMapper, EvaluationDeptBusinessPlan> implements IEvaluationDeptBusinessPlanService {

	@Autowired
	private IEvaluationDeptService evaluationDeptService;

	@Autowired
	private IEvaluationDeptIndicatorService evaluationDeptIndicatorService;

	@Autowired
	private IEvaluationDeptIndicatorItemService evaluationDeptIndicatorItemService;

	@Autowired
	private IEvaluationDeptPlanContentService evaluationDeptPlanContentService;

	@Override
	public IPage<EvaluationDeptBusinessPlan> listPage(Integer pageNum, Integer pageSize, String responsibilityCode, String des, String appraisalYear, String stage) {
		if (null == pageNum || null == pageSize) {
			throw new RuntimeException("分页参数为空");
		}
		//构造分页构造器
		Page<EvaluationDeptBusinessPlan> page = new Page<>(pageNum, pageSize);
		return baseMapper.selectListPage(page, responsibilityCode, des, appraisalYear, stage);
	}

	@Override
	public List<EvaluationDeptBusinessPlan> listAll(String responsibilityCode, String des, String appraisalYear) {
		QueryWrapper<EvaluationDeptBusinessPlan> qw = new QueryWrapper<>();

		if (StringUtils.isNotEmpty(responsibilityCode)) {
			qw.lambda().eq(EvaluationDeptBusinessPlan::getResponsibilityCode, responsibilityCode);
		}

		if (StringUtils.isNotEmpty(des)) {
			qw.lambda().eq(EvaluationDeptBusinessPlan::getDes, des);
		}

		if (StringUtils.isNotEmpty(appraisalYear)) {
			qw.lambda().eq(EvaluationDeptBusinessPlan::getAppraisalYear, appraisalYear);
		}

		return baseMapper.selectList(qw);
	}

	@Override
	public boolean save(EvaluationDeptBusinessPlan deptBusinessPlan) {
		String businessPlanId = Convert.toStr(IdUtil.getSnowflake(1, 1).nextId());
		deptBusinessPlan.setId(businessPlanId);
		deptBusinessPlan.setResponsibilityCode("bmkp_" + deptBusinessPlan.getBusinessClass().toLowerCase() + "_" + deptBusinessPlan.getAppraisalYear());
		deptBusinessPlan.setDocumentStatus("有效");
		deptBusinessPlan.setStage("流程未启动");
		deptBusinessPlan.setAssessmentStage("流程未启动");
		QueryWrapper<EvaluationDept> qw = new QueryWrapper<>();
		if (StringUtils.isNotEmpty(deptBusinessPlan.getAppraisalYear())) {
			qw.lambda().eq(EvaluationDept::getAnnual, deptBusinessPlan.getAppraisalYear());
		}
		List<EvaluationDept> evaluationDeptList = evaluationDeptService.list(qw);
		StringBuilder deptIds = new StringBuilder();
		StringBuilder deptNames = new StringBuilder();
		for (int i = 0; i < evaluationDeptList.size(); i++) {
			EvaluationDept evaluationDept = evaluationDeptList.get(i);
			List<EvaluationDeptIndicatorItem> evaluationDeptIndicatorItemList = evaluationDeptIndicatorItemService.selectEvaluationDeptIndicatorItemListByEvaluationDeptId(evaluationDept.getId());
			for (EvaluationDeptIndicatorItem evaluationDeptIndicatorItem : evaluationDeptIndicatorItemList) {
				EvaluationDeptIndicator evaluationDeptIndicator = evaluationDeptIndicatorService.getById(evaluationDeptIndicatorItem.getEvaluationDeptIndicatorId());
				EvaluationDeptPlanContent evaluationDeptPlanContent = new EvaluationDeptPlanContent();
				evaluationDeptPlanContent.setBusinessPlanId(businessPlanId);
				evaluationDeptPlanContent.setDeptId(evaluationDept.getDeptId());
				evaluationDeptPlanContent.setDeptName(evaluationDept.getDeptName());
				evaluationDeptPlanContent.setIndicatorId(evaluationDeptIndicator.getId());
				evaluationDeptPlanContent.setIndicatorName(evaluationDeptIndicator.getIndicatorName());
				evaluationDeptPlanContent.setIndicatorItemId(evaluationDeptIndicatorItem.getId());
				evaluationDeptPlanContent.setIndicatorItemName(evaluationDeptIndicatorItem.getIndicatorItemName());
				evaluationDeptPlanContent.setAuditStatus("-1");
				evaluationDeptPlanContentService.save(evaluationDeptPlanContent);
			}
			if (i == evaluationDeptList.size() - 1) {
				deptIds.append(evaluationDept.getDeptId());
				deptNames.append(evaluationDept.getDeptName());
			} else {
				deptIds.append(evaluationDept.getDeptId()).append(",");
				deptNames.append(evaluationDept.getDeptName()).append(",");
			}
		}
		deptBusinessPlan.setDeptId(deptIds.toString());
		deptBusinessPlan.setDeptName(deptNames.toString());
		return super.save(deptBusinessPlan);
	}

	@Override
	public boolean editState(String id, String state, String instId) {
		EvaluationDeptBusinessPlan evaluationDeptBusinessPlan = baseMapper.selectById(id);
		if (StringUtils.isNotEmpty(state)) {
			evaluationDeptBusinessPlan.setStage(state);
		}
		if (StringUtils.isNotEmpty(instId)) {
			evaluationDeptBusinessPlan.setInstId(instId);
		}
		return super.saveOrUpdate(evaluationDeptBusinessPlan);
	}

	@Override
	public boolean editAState(String id, String assessmentStage, String assessmentInstId) {
		EvaluationDeptBusinessPlan evaluationDeptBusinessPlan = baseMapper.selectById(id);
		if (StringUtils.isNotEmpty(assessmentStage)) {
			evaluationDeptBusinessPlan.setAssessmentStage(assessmentStage);
		}
		if (StringUtils.isNotEmpty(assessmentInstId)) {
			evaluationDeptBusinessPlan.setAssessmentInstId(assessmentInstId);
		}
		return super.saveOrUpdate(evaluationDeptBusinessPlan);
	}
}
